//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Fri Jan  8 08:33:57 PST 1999
// Last Modified: Fri Jan  8 08:34:01 PST 1999
// Last Modified: Tue Jun 29 16:18:02 PDT 1999 (added sysex capability)
// Filename:      ...sig/maint/code/control/MidiInPort/linux/MidiInPort_linux.h
// Web Address:   http://www-ccrma.stanford.edu/~craig/improv/include/MidiInPort_linux.h
// Syntax:        C++ 
//
// Description:   An interface for MIDI input capabilities of
//                linux OSS sound driver's specific MIDI input methods.
//                This class is inherited privately by the MidiInPort class.
//

#ifndef _MIDIINPORT_LINUX_H_INCLUDED
#define _MIDIINPORT_LINUX_H_INCLUDED

#ifdef LINUX

#include "MidiMessage.h"
#include "CircularBuffer.h"
#include "Array.h"
#include "Sequencer_linux.h"
#include "SigTimer.h"
#include <pthread.h>

typedef unsigned char uchar;


class MidiInPort_linux : public Sequencer_linux {
   public:
                      MidiInPort_linux           (void);
                      MidiInPort_linux           (int aPort, int autoOpen = 1);
                     ~MidiInPort_linux           ();

      void            clearSysex                 (int buffer);
      void            clearSysex                 (void);
      void            close                      (void);
      void            closeAll                   (void);
      MidiMessage     extract                    (void);
      int             getBufferSize              (void);
      int             getChannelOffset           (void) const;
      int             getCount                   (void);
      const char*     getName                    (void);
      static const char* getName                 (int i);
      static int      getNumPorts                (void);
      int             getPort                    (void);
      int             getPortStatus              (void);
      uchar*          getSysex                   (int buffer);
      int             getSysexSize               (int buffer);
      int             getTrace                   (void);
      void            insert                     (const MidiMessage& aMessage);
      int             installSysex               (uchar* anArray, int aSize);
      MidiMessage&    message                    (int index);
      int             open                       (void);
      void            pause                      (void);
      void            setBufferSize              (int aSize);
      void            setChannelOffset           (int anOffset);
      void            setPort                    (int aPort);
      int             setTrace                   (int aState);
      void            toggleTrace                (void);
      void            unpause                    (void);

   protected:
      int    port;     // the port to which this object belongs

      static int      installSysexPrivate        (int port, 
                                                    uchar* anArray, int aSize);
 
      static int        objectCount;     // num of similar objects in existence
      static int*       portObjectCount; // objects connected to particular port
      static int*       trace;           // for verifying input
      static ostream*   tracedisplay;    // stream for displaying trace
      static int        numDevices;      // number of input ports
      static CircularBuffer<MidiMessage>** midiBuffer; // MIDI storage frm ports
      static int        channelOffset;   // channel offset, either 0 or 1
                                         // not being used right now.
      static int*       pauseQ;          // for adding items to Buffer or not
      static SigTimer   midiTimer;       // for timing MIDI input
      static pthread_t  midiInThread;    // for MIDI input thread function
      static int*       sysexWriteBuffer; // for MIDI sysex write location
      static Array<uchar>** sysexBuffers; // for MIDI sysex storage

   private:
      void            deinitialize               (void); 
      void            initialize                 (void); 

 
   friend void *interpretMidiInputStreamPrivate(void * x);
    
};


#endif  // LINUX

#endif  // _MIDIINPORT_LINUX_H_INCLUDED



// md5sum:        bf2ea0c06f3756c78d46c4464a786c49  MidiInPort_linux.h
